@@ -10,8 +10,8 @@
 /*                                                                          */
 /****************************************************************************/
 
-#ifndef _FT_COMMON_H_
-#define _FT_COMMON_H_
+#ifndef FT_COMMON_H_
+#define FT_COMMON_H_
 
 
 #include <ft2build.h>
@@ -65,6 +65,9 @@
   /* baseline distance between header lines */
 #define HEADER_HEIGHT  12
 
+  /* default gamma setting */
+#define GAMMA  1.8
+
 #include "graph.h"
 #include "grobjs.h"
 #include "grfont.h"
@@ -162,7 +165,6 @@
     FT_Fixed    center;            /* 0..1 */
     int         vertical;          /* displayed vertically? */
     FT_Matrix*  matrix;            /* string transformation */
-    FT_Byte*    gamma_ramp;        /* an array of size 256 */
 
   } FTDemo_String_Context;
 
@@ -336,6 +338,6 @@
   FTDemo_Make_Encoding_Tag( const char*  s );
 
 
-#endif /* _FTCOMMON_H_ */
+#endif /* FTCOMMON_H_ */
 
 /* End */
