@@ -85,7 +85,7 @@
 
   /** DISPLAY ABSTRACTION **/
 
-  typedef enum  _DisplayMode
+  typedef enum  DisplayMode_
   {
     DISPLAY_MODE_MONO = 0,
     DISPLAY_MODE_GRAY,
@@ -111,7 +111,7 @@
                        const char*  msg );
 
 
-  typedef struct  _DisplayRec
+  typedef struct  DisplayRec_
   {
     void*             disp;
     Display_drawFunc  disp_draw;
@@ -150,7 +150,7 @@
   /***********************************************************************/
   /***********************************************************************/
 
-  typedef enum  _HintMode
+  typedef enum  HintMode_
   {
     HINT_MODE_UNHINTED,
     HINT_MODE_AUTOHINT,
@@ -172,7 +172,7 @@
 
   /** RENDER STATE **/
 
-  typedef struct  _ColumnStateRec
+  typedef struct  ColumnStateRec_
   {
     int            use_cboxes;
     int            use_kerning;
@@ -193,7 +193,7 @@
   } ColumnStateRec, *ColumnState;
 
 
-  typedef struct  _FontFaceRec
+  typedef struct  FontFaceRec_
   {
     const char*  filepath;
     char*        family_name;
@@ -203,7 +203,7 @@
   } FontFaceRec, *FontFace;
 
 
-  typedef struct  _RenderStateRec
+  typedef struct  RenderStateRec_
   {
     FT_Library      library;
     const char*     text;
@@ -270,7 +270,7 @@
     state->columns[0].use_custom_lcd_filter  = 0;
     state->columns[0].fw_index               = 2;
     /* freetype default filter weights */
-    memcpy( state->columns[0].filter_weights, "\x10\x40\x70\x40\x10", 5 );
+    memcpy( state->columns[0].filter_weights, "\x08\x4D\x56\x4D\x08", 5 );
 
     state->columns[1]                        = state->columns[0];
     state->columns[1].hint_mode              = HINT_MODE_AUTOHINT;
@@ -873,7 +873,7 @@
 #include "grobjs.h"
 #include "grfont.h"
 
-  typedef struct  _ADisplayRec
+  typedef struct  ADisplayRec_
   {
     int         width;
     int         height;
@@ -916,7 +916,7 @@
     display->height  = height;
     display->surface = surface;
     display->bitmap  = &surface->bitmap;
-    display->gamma   = 1.0;
+    display->gamma   = GAMMA;
 
     grSetGlyphGamma( display->gamma );
 
@@ -1368,7 +1368,7 @@
   write_global_info( RenderState  state )
   {
     ADisplay  adisplay = (ADisplay)state->display.disp;
-    double    gamma    = adisplay->gamma;
+    char      gamma[] = "sRGB";
     char      buf[256];
 
     FontFace     face = &state->faces[state->face_index];
@@ -1383,12 +1383,13 @@
     grWriteCellString( adisplay->bitmap, 0, 5,
                        buf, adisplay->fore_color );
 
-    sprintf( buf, "%.1fpt (%dppem) at %ddpi, gamma: %.1f%s",
+    if ( adisplay->gamma != 0.0 )
+      sprintf( gamma, "%.1f", adisplay->gamma );
+    sprintf( buf, "%.1fpt (%dppem) at %ddpi, gamma: %s",
                   state->char_size,
                   (int)(state->char_size * state->resolution / 72 + 0.5),
                   state->resolution,
-                  gamma,
-                  gamma == 0.0 ? " (sRGB)" : "" );
+                  gamma );
     grWriteCellString( adisplay->bitmap, 0, 5 + HEADER_HEIGHT,
                        buf, adisplay->fore_color );
 
