@@ -2,7 +2,7 @@
 /*                                                                          */
 /*  The FreeType project -- a free and portable quality TrueType renderer.  */
 /*                                                                          */
-/*  Copyright 1996-2007, 2009-2014 by                                       */
+/*  Copyright 1996-2007, 2009-2015 by                                       */
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
 /*                                                                          */
@@ -20,6 +20,8 @@
 #include <stdarg.h>
 #include <math.h>
 
+#include FT_LCD_FILTER_H
+
 #define CELLSTRING_HEIGHT  8
 #define MAXPTSIZE          500   /* dtp */
 
@@ -47,14 +49,14 @@
 
     FTDemo_String_Context  sc;
 
-    FT_Byte      gamma_ramp[256];
+    FT_Byte      gamma_ramp[256];   /* for show only */
     FT_Matrix    trans_matrix;
     int          font_index;
     char*        header;
     char         header_buffer[256];
 
   } status = { DIM_X, DIM_Y,
-               RENDER_MODE_STRING, FT_ENCODING_UNICODE, 72, 48, 2.0, 0 };
+               RENDER_MODE_STRING, FT_ENCODING_UNICODE, 72, 48, GAMMA, 0 };
 
   static FTDemo_Display*  display;
   static FTDemo_Handle*   handle;
@@ -112,11 +114,12 @@
     grWriteln( "  h         : toggle outline hinting" );
     grLn();
     grWriteln( "  1-2       : select rendering mode" );
+    grWriteln( "  l         : cycle through LCD modes" );
     grWriteln( "  k         : cycle through kerning modes" );
     grWriteln( "  t         : cycle through kerning degrees" );
+    grWriteln( "  space     : cycle through color" );
     grWriteln( "  V         : toggle vertical rendering" );
     grLn();
-    grWriteln( "  G         : toggle gamma correction" );
     grWriteln( "  g         : increase gamma by 0.1" );
     grWriteln( "  v         : decrease gamma by 0.1" );
     grLn();
@@ -165,7 +168,12 @@
     FT_Fixed  sinus;
 
 
-    status.angle = ( status.angle + delta ) % 360;
+    status.angle += delta;
+
+    if ( status.angle <= -180 )
+      status.angle += 360;
+    if ( status.angle > 180 )
+      status.angle -= 360;
 
     if ( status.angle == 0 )
     {
@@ -176,10 +184,7 @@
 
     status.sc.matrix = &status.trans_matrix;
 
-    if ( status.angle < 0 )
-      status.angle += 360;
-
-    radian  = status.angle * 3.14159 / 180.0;
+    radian  = status.angle * 3.14159265 / 180.0;
     cosinus = (FT_Fixed)( cos( radian ) * 65536.0 );
     sinus   = (FT_Fixed)( sin( radian ) * 65536.0 );
 
@@ -190,6 +195,57 @@
   }
 
 
+  static void
+  event_lcdmode_change( void )
+  {
+    const char  *lcd_mode;
+
+
+    handle->lcd_mode++;
+
+    switch ( handle->lcd_mode )
+    {
+    case LCD_MODE_LIGHT:
+      lcd_mode = " light AA";
+      break;
+    case LCD_MODE_RGB:
+      lcd_mode = " LCD (horiz. RGB)";
+      break;
+    case LCD_MODE_BGR:
+      lcd_mode = " LCD (horiz. BGR)";
+      break;
+    case LCD_MODE_VRGB:
+      lcd_mode = " LCD (vert. RGB)";
+      break;
+    case LCD_MODE_VBGR:
+      lcd_mode = " LCD (vert. BGR)";
+      break;
+    default:
+      handle->lcd_mode = 0;
+      lcd_mode = " normal AA";
+    }
+
+    sprintf( status.header_buffer, "mode changed to %s", lcd_mode );
+    status.header = status.header_buffer;
+  }
+
+
+  static void
+  event_color_change( void )
+  {
+    static int     i = 0;
+    unsigned char  r = i & 4 ? 0xff : 0;
+    unsigned char  g = i & 2 ? 0xff : 0;
+    unsigned char  b = i & 1 ? 0xff : 0;
+
+
+    display->back_color = grFindColor( display->bitmap,  r,  g,  b, 0xff );
+    display->fore_color = grFindColor( display->bitmap, ~r, ~g, ~b, 0xff );
+
+    i++;
+  }
+
+
   static void
   event_gamma_change( double  delta )
   {
@@ -204,14 +260,13 @@
     else if ( status.gamma < 0.1 )
       status.gamma = 0.1;
 
-    sprintf( status.header_buffer, "gamma changed to %.1f", status.gamma );
-    status.header = status.header_buffer;
+    grSetGlyphGamma( status.gamma );
 
-    gamma_inv = 1.0f / status.gamma;
+    gamma_inv = 1. / status.gamma;
 
     for ( i = 0; i < 256; i++ )
-      status.gamma_ramp[i] = (FT_Byte)( pow( (double)i / 255.0f, gamma_inv )
-                                        * 255.0f );
+      status.gamma_ramp[i] = (FT_Byte)( pow( (double)i / 255., gamma_inv )
+                                        * 255. + 0.5 );
   }
 
 
@@ -316,6 +371,12 @@
       FTDemo_Update_Current_Flags( handle );
       break;
 
+    case grKEY( 'l' ):
+      event_lcdmode_change();
+
+      FTDemo_Update_Current_Flags( handle );
+      break;
+
     case grKEY( 'k' ):
       sc->kerning_mode = ( sc->kerning_mode + 1 ) % N_KERNING_MODES;
       status.header =
@@ -338,6 +399,10 @@
             : (char *)"tight track kerning active";
       break;
 
+    case grKeySpace:
+      event_color_change();
+      break;
+
     case grKEY( 'V' ):
       sc->vertical  = !sc->vertical;
       status.header = sc->vertical
@@ -345,13 +410,6 @@
                       : (char *)"using horizontal layout";
       break;
 
-    case grKEY( 'G' ):
-      sc->gamma_ramp = sc->gamma_ramp ? NULL : status.gamma_ramp;
-      status.header  = sc->gamma_ramp
-                       ? (char *)"gamma correction is now on"
-                       : (char *)"gamma correction is now off";
-      break;
-
     case grKEY( 'g' ):
       event_gamma_change( 0.1 );
       break;
@@ -391,6 +449,7 @@
                    grBitmap*  bitmap )
   {
     int       i, x, y;
+    int       bpp = bitmap->pitch / bitmap->width;
     FT_Byte*  p = (FT_Byte*)bitmap->buffer;
 
 
@@ -401,7 +460,7 @@
     y = ( bitmap->rows + 256 ) / 2;
 
     for (i = 0; i < 256; i++)
-      p[bitmap->pitch * ( y - gamma_ramp[i] ) + ( x + i )] = 80;
+      p[bitmap->pitch * ( y - gamma_ramp[i] ) + bpp * ( x + i )] ^= 0xFF;
   }
 
 
@@ -451,8 +510,8 @@
     grWriteCellString( display->bitmap, 0, 0,
                        status.header, display->fore_color );
 
-    sprintf( status.header_buffer, "at %g points, angle = %d",
-             status.ptsize / 64.0, status.angle );
+    sprintf( status.header_buffer, "at %g points, angle = %d, gamma = %g",
+             status.ptsize / 64.0, status.angle, status.gamma );
     grWriteCellString( display->bitmap, 0, CELLSTRING_HEIGHT,
                        status.header_buffer, display->fore_color );
 
@@ -478,7 +537,7 @@
       "            ppem value (pixels per EM).\n" );
     fprintf( stderr,
       "  font      The font file(s) to display.\n"
-      "            For Type 1 font files, ftview also tries to attach\n"
+      "            For Type 1 font files, ftstring also tries to attach\n"
       "            the corresponding metrics file (with extension\n"
       "            `.afm' or `.pfm').\n"
       "\n" );
@@ -596,6 +655,8 @@
 
     parse_cmdline( &argc, &argv );
 
+    FT_Library_SetLcdFilter( handle->library, FT_LCD_FILTER_LIGHT );
+
     handle->encoding  = status.encoding;
     handle->use_sbits = 0;
     FTDemo_Update_Current_Flags( handle );
@@ -617,10 +678,8 @@
     if ( handle->num_fonts == 0 )
       PanicZ( "could not open any font file" );
 
-    display = FTDemo_Display_New( gr_pixel_mode_gray,
+    display = FTDemo_Display_New( gr_pixel_mode_rgb24,
                                   status.width, status.height );
-    display->back_color.value = 0;
-    display->fore_color.value = 0xff;
 
     if ( !display )
       PanicZ( "could not allocate display surface" );
@@ -628,14 +687,17 @@
     grSetTitle( display->surface,
                 "FreeType String Viewer - press ? for help" );
 
+    event_color_change();
     event_gamma_change( 0 );
     event_font_change( 0 );
     status.header = 0;
 
-    for ( ;; )
+    do
     {
       FTDemo_Display_Clear( display );
 
+      gamma_ramp_draw( status.gamma_ramp, display->bitmap );
+
       switch ( status.render_mode )
       {
       case RENDER_MODE_STRING:
@@ -693,16 +755,11 @@
         break;
       }
 
-      if ( !error && status.sc.gamma_ramp )
-        gamma_ramp_draw( status.gamma_ramp, display->bitmap );
-
       write_header( error );
 
       status.header = 0;
       grListenSurface( display->surface, 0, &event );
-      if ( Process_Event( &event ) )
-        break;
-    }
+    } while ( !Process_Event( &event ) );
 
     printf( "Execution completed successfully.\n" );
 
